import os
import time
from typing import List

import click
import vessl

from vessl.cli._base import VesslGroup, vessl_argument, vessl_option
from vessl.cli._util import (
    Endpoint,
    generic_prompter,
    print_data,
    print_logs,
    print_table,
    print_volume_files,
    prompt_choices,
    truncate_datetime,
)
from vessl.cli.kernel_cluster import cluster_option
from vessl.cli.kernel_resource_spec import resource_option
from vessl.cli.organization import organization_name_option
from vessl.cli.project import project_name_option
from vessl.experiment import (
    create_experiment,
    download_experiment_output_files,
    list_experiment_logs,
    list_experiment_output_files,
    list_experiments,
    read_experiment,
)
from vessl.kernel_cluster import list_cluster_nodes
from vessl.kernel_image import list_kernel_images
from vessl.project import _get_project
from vessl.util.constant import (
    FRAMEWORK_TYPE_PYTORCH,
    FRAMEWORK_TYPE_TENSORFLOW,
    FRAMEWORK_TYPES,
    MOUNT_PATH_OUTPUT,
    PROCESSOR_TYPE_GPU,
    PROCESSOR_TYPES,
)
from vessl.util.exception import GitError
from vessl.util.git import get_git_repo


def experiment_name_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
) -> str:
    experiments = list_experiments()
    return prompt_choices(
        "Experiment", [(f"{x.name} #{x.number}", x.name) for x in reversed(experiments)]
    )


def local_experiment_name_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
) -> str:
    experiments = [e for e in list_experiments() if e.is_local]
    return prompt_choices(
        "Experiment", [(f"{x.name} #{x.number}", x.name) for x in reversed(experiments)]
    )


def processor_type_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before processor type (`--processor-type`).",
        )

    if ctx.obj.get("resource") is None:
        processor_type = prompt_choices("Processor Type", PROCESSOR_TYPES)
        ctx.obj["processor_type"] = processor_type
        return processor_type


def processor_type_callback(
    ctx: click.Context, param: click.Parameter, value: str
):
    if value:
        ctx.obj["processor_type"] = value
    return value


def cpu_limit_prompter(
    ctx: click.Context, param: click.Parameter, value: float
) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before CPU limit (`--cpu-limit`).",
        )

    if ctx.obj.get("resource") is None:
        return click.prompt("CPUs (in vCPU)", type=click.FLOAT)


def memory_limit_prompter(
    ctx: click.Context, param: click.Parameter, value: float
) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before memory limit (`--memory-limit`).",
        )

    if ctx.obj.get("resource") is None:
        return click.prompt("Memory (e.g. 4Gi)", type=click.STRING)


def gpu_type_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before GPU type (`--gpu-type`).",
        )

    if ctx.obj.get("resource") is None:
        processor_type = ctx.obj.get("processor_type")
        if processor_type is None:
            raise click.UsageError(
                message="Processor type must be specified before GPU type (`--gpu-type`).",
            )
        if processor_type == PROCESSOR_TYPE_GPU:
            nodes = list_cluster_nodes(cluster.id)
            return prompt_choices("GPU Type", [x.gpu_product_name for x in nodes])


def gpu_limit_prompter(
    ctx: click.Context, param: click.Parameter, value: float
) -> float:
    cluster = ctx.obj.get("cluster")
    if cluster is None:
        raise click.BadOptionUsage(
            option_name="--cluster",
            message="Cluster (`--cluster`) must be specified before GPU limit (`--gpu-limit`).",
        )

    if ctx.obj.get("resource") is None:
        processor_type = ctx.obj.get("processor_type")
        if processor_type is None:
            raise click.UsageError(
                message="Processor type must be specified before GPU limit (`--gpu-limit`).",
            )

        if processor_type == PROCESSOR_TYPE_GPU:
            return click.prompt("GPUs (in vGPU)", type=click.FLOAT)


def image_url_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    processor_type = ctx.obj.get("processor_type")
    if processor_type is None:
        raise click.UsageError(
            message="Processor type must be specified before image URL (`--image-url`).",
        )

    images = list_kernel_images()
    images = [x for x in images if x.processor_type == processor_type]

    return prompt_choices("Image URL", [x.image_url for x in images])


def upload_local_git_diff_prompter(ctx: click.Context, param: click.Parameter, value: bool) -> bool:
    try:
        local_git_owner, local_git_repo = get_git_repo()
    except GitError:
        return False

    if ctx.params.get("git_diff"):
        return False

    project = _get_project(**ctx.params)
    for repo in project.project_repositories:
        if repo.git_owner == local_git_owner and repo.git_repo == local_git_repo:
            return click.prompt("Upload local git commit & diff (y/n)", type=click.BOOL)
    return False


def worker_count_callback(
    ctx: click.Context, param: click.Parameter, value: int
) -> int:
    if value is None:
        value = 1

    if value < 1:
        raise click.BadOptionUsage(
            option_name="--worker-count",
            message="num nodes (`--num-nodes`) must be a positive integer.",
        )

    ctx.obj["worker_count"] = value
    return value


def framework_type_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    worker_count = ctx.obj.get("worker_count")
    if worker_count == 1:
        return ""

    framework_type = prompt_choices("Processor Type", FRAMEWORK_TYPES)
    if framework_type == FRAMEWORK_TYPE_TENSORFLOW:
        raise click.BadOptionUsage(
            option_name="--framework-type",
            message="Only PyTorch distributed experiment is supported currently.",
        )
    return framework_type


@click.command(name="experiment", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=experiment_name_prompter
)
@organization_name_option
@project_name_option
def read(name: str):
    experiment = read_experiment(experiment_name_or_number=name)

    distributed_spec = "None"
    if experiment.is_distributed:
        if experiment.distributed_spec.framework_type == FRAMEWORK_TYPE_PYTORCH:
            distributed_spec = {
                "Framework Type": experiment.distributed_spec.framework_type,
                "PyTorch Spec": {
                    "Worker count": experiment.distributed_spec.pytorch_spec.worker_replicas,
                },
            }
        elif experiment.distributed_spec.framework_type == FRAMEWORK_TYPE_TENSORFLOW:
            distributed_spec = {
                "Framework Type": experiment.distributed_spec.framework_type,
                "TensorFlow Spec": experiment.distributed_spec.tensorflow_spec,
            }

    kernel_image = "None"
    if experiment.kernel_image:
        kernel_image = {
            "Name": experiment.kernel_image.name,
            "URL": experiment.kernel_image.image_url,
        }

    resource_spec = "None"
    if experiment.kernel_resource_spec:
        resource_spec = {
            "Name": experiment.kernel_resource_spec.name,
            "CPU Type": experiment.kernel_resource_spec.cpu_type,
            "CPU Limit": experiment.kernel_resource_spec.cpu_limit,
            "Memory Limit": experiment.kernel_resource_spec.memory_limit,
            "GPU Type": experiment.kernel_resource_spec.gpu_type,
            "GPU Limit": experiment.kernel_resource_spec.gpu_limit,
        }

    print_data(
        {
            "ID": experiment.id,
            "Number": experiment.number,
            "Name": experiment.name,
            "Distributed": experiment.is_distributed,
            "Distributed Spec": distributed_spec,
            "Local": experiment.is_local,
            "Status": experiment.status,
            "Created": truncate_datetime(experiment.created_dt),
            "Message": experiment.message,
            "Kernel Image": kernel_image,
            "Resource Spec": resource_spec,
            "Start command": experiment.start_command,
        }
    )
    print(
        f"For more info: {Endpoint.experiment.format(experiment.organization.name, experiment.project.name, experiment.number)}"
    )


@cli.vessl_command()
@organization_name_option
@project_name_option
def list():
    experiments = list_experiments()
    print_table(
        experiments,
        ["ID", "Name", "Distributed", "Status", "Created", "Message"],
        lambda x: [
            x.id,
            x.name,
            x.is_distributed,
            x.status,
            truncate_datetime(x.created_dt),
            x.message,
        ],
    )


command_option = vessl_option(
    "-x",
    "--command",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Start command"),
    help="Start command to execute in experiment container.",
)
processor_type_option = vessl_option(
    "--processor-type",
    type=click.Choice(("CPU", "GPU")),
    prompter=processor_type_prompter,
    callback=processor_type_callback,
    help="CPU or GPU (for custom resource only).",
)
cpu_limit_option = vessl_option(
    "--cpu-limit",
    type=click.FLOAT,
    prompter=cpu_limit_prompter,
    help="Number of vCPUs (for custom resource only).",
)
memory_limit_option = vessl_option(
    "--memory-limit",
    type=click.STRING,
    prompter=memory_limit_prompter,
    help="Memory limit (e.g. 4Gi) (for custom resource only).",
)
gpu_type_option = vessl_option(
    "--gpu-type",
    type=click.STRING,
    prompter=gpu_type_prompter,
    help="GPU type such as Tesla-K80 (for custom resource only).",
)
gpu_limit_option = vessl_option(
    "--gpu-limit",
    type=click.INT,
    prompter=gpu_limit_prompter,
    help="Number of GPU cores (for custom resource only).",
)
image_url_option = vessl_option(
    "-i",
    "--image-url",
    type=click.STRING,
    prompter=image_url_prompter,
    help="Kernel docker image URL",
)
message_option = click.option("-m", "--message", type=click.STRING)
termination_protection_option = click.option("--termination-protection", is_flag=True)
env_var_option = click.option(
    "-e",
    "--env-var",
    type=click.STRING,
    multiple=True,
    help="Environment variables. Format: [key]=[value], ex. `--env-var PORT=8080`.",
)
dataset_option = click.option(
    "--dataset",
    type=click.STRING,
    multiple=True,
    help="Dataset mounts. Format: [mount_path]:[dataset_name]@[optional_dataset_version], ex. `--dataset /input:mnist@3bcd5f`.",
)
git_ref_option = click.option(
    "--git-ref",
    type=click.STRING,
    multiple=True,
    help="Git repository mounts. Format: [mount_path]:github/[organization]/[repository]/[optional_commit], ex. `--git-ref /home/vessl/examples:github/savvihub/examples/3cd23dd`.",
)
git_diff_option = click.option(
    "--git-diff",
    type=click.STRING,
    help="Git diff file mounts. Format: [mount_path]:[volume_file_path]]. This option is used only for reproducing existing experiments.",
)
upload_local_file_option = click.option(
    "--upload-local-file",
    type=click.STRING,
    multiple=True,
    help="Upload local file. Format: [local_path] or [local_path]:[remote_path].",
)
upload_local_git_diff_option = vessl_option(
    "--upload-local-git-diff",
    type=click.BOOL,
    prompter=upload_local_git_diff_prompter,
    help="Upload local git commit hash and diff. Format: --upload-local-git-diff=true or --upload-local-git-diff=false",
)
archive_file_option = click.option(
    "--archive-file",
    type=click.STRING,
    help="Local archive file mounts. Format: [mount_path]:[archive_file_path]]. This option is used only for reproducing existing experiments.",
)
root_volume_size_option = click.option("--root-volume-size", type=click.STRING)
working_dir_option = click.option(
    "--working-dir", type=click.STRING, help="Defaults to `/home/vessl/`."
)
output_dir_option = click.option(
    "--output-dir",
    type=click.STRING,
    default=MOUNT_PATH_OUTPUT,
    help="Directory to store experiment output files. Defaults to `/output`.",
)
worker_count_option = vessl_option(
    "--worker-count",
    type=click.INT,
    callback=worker_count_callback,
    help="The number of nodes to run an experiment. Defaults to 1",
)
framework_type_option = vessl_option(
    "--framework-type",
    type=click.STRING,
    prompter=framework_type_prompter,
    help="Framework type option. Defaults to `pytorch`.",
)


@cli.vessl_command()
@click.pass_context
@organization_name_option
@project_name_option
@cluster_option
@resource_option
@processor_type_option
@cpu_limit_option
@memory_limit_option
@gpu_type_option
@gpu_limit_option
@image_url_option
@message_option
@termination_protection_option
@env_var_option
@dataset_option
@git_ref_option
@git_diff_option
@upload_local_file_option
@upload_local_git_diff_option
@archive_file_option
@root_volume_size_option
@working_dir_option
@output_dir_option
@command_option
@worker_count_option
@framework_type_option
def create(
    ctx,
    cluster: str,
    command: str,
    resource: str,
    processor_type: str,
    cpu_limit: float,
    memory_limit: str,
    gpu_type: str,
    gpu_limit: int,
    image_url: str,
    message: str,
    termination_protection: bool,
    env_var: List[str],
    dataset: List[str],
    git_ref: List[str],
    git_diff: str,
    upload_local_file: List[str],
    upload_local_git_diff: bool,
    archive_file: str,
    root_volume_size: str,
    working_dir: str,
    output_dir: str,
    worker_count: int,
    framework_type: str,
):
    experiment = create_experiment(
        cluster_name=cluster,
        start_command=command,
        kernel_resource_spec_name=resource,
        processor_type=processor_type,
        cpu_limit=cpu_limit,
        memory_limit=memory_limit,
        gpu_type=gpu_type,
        gpu_limit=gpu_limit,
        kernel_image_url=image_url,
        message=message,
        termination_protection=termination_protection,
        env_vars=env_var,
        dataset_mounts=dataset,
        git_ref_mounts=git_ref,
        git_diff_mount=git_diff,
        local_files=upload_local_file,
        upload_local_git_diff=upload_local_git_diff,
        archive_file_mount=archive_file,
        root_volume_size=root_volume_size,
        working_dir=working_dir,
        output_dir=output_dir,
        worker_count=worker_count,
        framework_type=framework_type,
    )
    ctx.obj["experiment"] = experiment
    print(
        f"Created '{experiment.name}'.\n"
        f"For more info: {Endpoint.experiment.format(experiment.organization.name, experiment.project.name, experiment.number)}"
    )


worker_number_option = vessl_option(
    "--worker-number",
    type=click.INT,
    default=0,
    help="Worker number (for distributed experiment only).",
)


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=experiment_name_prompter
)
@click.option(
    "--tail",
    type=click.INT,
    default=200,
    help="Number of lines to display (from the end).",
)
@click.option(
    "-f",
    "--follow",
    is_flag=True,
    default=False,
)
@organization_name_option
@project_name_option
@worker_number_option
def logs(
    name: str,
    tail: int,
    follow: bool,
    worker_number: int,
):
    if not follow:
        logs = list_experiment_logs(
            experiment_name=name, tail=tail, worker_numer=worker_number
        )
        print_logs(logs)
        print(f"Displayed last {len(logs)} lines of '{name}'.")
        return

    after = 0
    while True:
        if read_experiment(name).status not in ["pending", "running"]:
            break

        logs = list_experiment_logs(
            experiment_name=name, after=after, worker_numer=worker_number
        )
        print_logs(logs)
        if len(logs) > 0:
            after = logs[-1].timestamp+0.000001
        time.sleep(3)


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=experiment_name_prompter
)
@organization_name_option
@project_name_option
@worker_number_option
def list_output(
    name: str,
    worker_number: int,
):
    files = list_experiment_output_files(
        experiment_name=name,
        need_download_url=False,
        recursive=True,
        worker_number=worker_number,
    )
    print_volume_files(files)


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=experiment_name_prompter
)
@click.option(
    "-p",
    "--path",
    type=click.Path(),
    default=os.path.join(os.getcwd(), "output"),
    help="Path to store downloads. Defaults to `./output`.",
)
@organization_name_option
@project_name_option
@worker_number_option
def download_output(
    name: str,
    path: str,
    worker_number: int,
):
    download_experiment_output_files(
        experiment_name=name,
        dest_path=path,
        worker_number=worker_number,
    )
    print(f"Downloaded experiment output to {path}.")



import os
from importlib.machinery import SourceFileLoader

import setuptools

version = (
    SourceFileLoader(
        "vessl._version",
        os.path.join(
            "vessl",
            "_version.py",
        ),
    )
    .load_module()
    .__VERSION__
)

with open("README.md") as fh:
    long_description = fh.read()

requirements = [
    "boto3",
    "click==8.0.1",
    "inquirer>=2.7.0",
    "paramiko==2.7.2",
    "python-dateutil>=2.8.1",
    "requests>=2.0.0",
    "requests-futures>=1.0.0",
    "sentry-sdk>=1.1.0",
    "schema>=0.7.4",
    "shortuuid",
    "sshpubkeys==3.3.1",
    "terminaltables>=3.1.0",
    "timeago==1.0.15",
    "toml==0.10.1",
    "tqdm",
    "psutil==5.8.0",
    "pynvml==11.0.0",
]

media_requirements = ["numpy", "Pillow>=8.0.0", "soundfile"]

setuptools.setup(
    name="vessl",
    version=version,
    author="Vessl AI Dev Team",
    author_email="contact@vessl.ai",
    description="A library and CLI for Vessl",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    entry_points={
        "console_scripts": [
            "vessl=vessl.cli._main:cli",
            "vsl=vessl.cli._main:cli",
        ],
    },
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    extras_require={"media": media_requirements},
)



import re
import shutil
import subprocess
import tempfile
from pathlib import Path

from vessl.util.exception import GitError


class GitRepository:
    def __init__(self):
        self.remote = "origin"
        if shutil.which("git"):
            self.root_path = self._get_root_path()
            self.owner, self.repo, self.remote = self._get_github_repo()
            self.commit_ref, self.branch, self.is_head = self._get_remote_revision()

    def check_revision_in_remote(self, revision):
        remote_branches = (
            subprocess.check_output(["git", "branch", "-r", "--contains", revision])
            .decode("utf-8")
            .strip()
            .split("\n")
        )
        for remote_branch in remote_branches:
            if remote_branch.startswith(f"{self.remote}/"):
                return True
        return False

    @staticmethod
    def get_commit_message(revision, format="%h %s (%cr) <%an>"):
        return (
            subprocess.check_output(
                ["git", "log", "--format=%s" % format, "-n", "1", revision]
            )
            .decode("utf-8")
            .strip()
        )

    @staticmethod
    def get_current_diff_status(revision_or_branch):
        untracked_files = (
            subprocess.check_output(["git", "ls-files", "-o", "--exclude-standard"])
            .decode("utf-8")
            .strip()
            .split("\n")
        )
        uncommitted_files = (
            subprocess.check_output(["git", "diff", "--stat", revision_or_branch])
            .decode("utf-8")
            .strip()
            .split("\n")
        )

        untracked_files = [x for x in untracked_files if len(x) > 0]
        uncommitted_files = [x for x in uncommitted_files if len(x) > 0]

        return len(untracked_files) > 0 or len(uncommitted_files) > 0, {
            "untracked": untracked_files,
            "uncommitted": uncommitted_files,
        }

    @staticmethod
    def get_current_diff_file(revision_or_branch, with_untracked=True):
        fp = tempfile.NamedTemporaryFile(suffix=".patch")

        untracked_files = []
        if with_untracked:
            untracked_files = (
                subprocess.check_output(["git", "ls-files", "-o", "--exclude-standard"])
                .decode("utf-8")
                .strip()
                .split("\n")
            )
            untracked_files = [x for x in untracked_files if len(x) > 0]
            for f in untracked_files:
                subprocess.check_output(["git", "add", "-N", f])

        subprocess.call(
            ["git", "diff", "-p", "--binary", f"{revision_or_branch}"], stdout=fp
        )

        if with_untracked:
            for f in untracked_files:
                subprocess.check_output(["git", "reset", "--", f])

        return fp

    @staticmethod
    def _get_root_path():
        try:
            return (
                subprocess.check_output(
                    ["git", "rev-parse", "--show-toplevel"], stderr=subprocess.DEVNULL
                )
                .decode("utf-8")
                .strip()
            )
        except subprocess.CalledProcessError:
            raise GitError("Are you running in a git repository?")

    @staticmethod
    def _get_github_repo():  # -> (owner, repo, remote)
        remotes = (
            subprocess.check_output(["git", "remote"])
            .decode("utf-8")
            .strip()
            .split("\n")
        )
        for remote in remotes:
            try:
                remote_url = (
                    subprocess.check_output(
                        ["git", "remote", "get-url", remote], stderr=subprocess.DEVNULL
                    )
                    .strip()
                    .decode("utf-8")
                )
                if "github.com" not in remote_url:
                    continue
            except subprocess.CalledProcessError:
                raise GitError(
                    "github.com is not found in remote repositories. You should add your repo to github first."
                )

            regex = re.compile(
                r"((git|ssh|http(s)?)|(git@[\w\.]+))(:(//)?)([\w\.@\:/\-~]+)(\.git)?(/)?"
            )
            repo = regex.search(remote_url).group(7).split("/")

            return repo[-2], repo[-1].rsplit(".git", 1)[0], remote

        return None, None, None

    def _get_active_branch_name(self):
        head_dir = Path(self.root_path) / ".git" / "HEAD"
        with head_dir.open("r") as f:
            content = f.read().splitlines()

        for line in content:
            if line[0:4] == "ref:":
                return line.partition("refs/heads/")[2]

    def _get_remote_revision(self):
        try:
            revision = (
                subprocess.check_output(
                    ["git", "rev-parse", "HEAD"], stderr=subprocess.DEVNULL
                )
                .decode("utf-8")
                .strip()
            )
        except subprocess.CalledProcessError:
            raise GitError(
                "git rev-parse HEAD failed. Are you running in git repository?"
            )

        if self.check_revision_in_remote(revision):
            # with revision and patch
            return revision, "HEAD", True
        else:
            # with remote branch and patch
            try:
                upstream_branch_name = (
                    subprocess.check_output(
                        [
                            "git",
                            "rev-parse",
                            "--abbrev-ref",
                            "--symbolic-full-name",
                            "@{u}",
                        ],
                        stderr=subprocess.DEVNULL,
                    )
                    .decode()
                    .strip()
                )
            except subprocess.CalledProcessError:
                raise GitError(
                    f"You should push your branch <{self._get_active_branch_name()}> to github first!"
                )

            revision = (
                subprocess.check_output(["git", "rev-parse", upstream_branch_name])
                .decode("utf-8")
                .strip()
            )
            return revision, upstream_branch_name, False



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseModelDetail(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'artifact_volume_id': 'int',
        'created_by': 'ResponseUser',
        'created_dt': 'datetime',
        'description': 'str',
        'experiment': 'ResponseSimpleExperiment',
        'id': 'int',
        'metrics_summary': 'OrmExperimentMetricsSummary',
        'status': 'str',
        'tags': 'list[ResponseTagResponse]',
        'updated_dt': 'datetime',
        'version': 'int'
    }

    attribute_map = {
        'artifact_volume_id': 'artifact_volume_id',
        'created_by': 'created_by',
        'created_dt': 'created_dt',
        'description': 'description',
        'experiment': 'experiment',
        'id': 'id',
        'metrics_summary': 'metrics_summary',
        'status': 'status',
        'tags': 'tags',
        'updated_dt': 'updated_dt',
        'version': 'version'
    }

    def __init__(self, artifact_volume_id=None, created_by=None, created_dt=None, description=None, experiment=None, id=None, metrics_summary=None, status=None, tags=None, updated_dt=None, version=None, local_vars_configuration=None):  # noqa: E501
        """ResponseModelDetail - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._artifact_volume_id = None
        self._created_by = None
        self._created_dt = None
        self._description = None
        self._experiment = None
        self._id = None
        self._metrics_summary = None
        self._status = None
        self._tags = None
        self._updated_dt = None
        self._version = None
        self.discriminator = None

        self.artifact_volume_id = artifact_volume_id
        self.created_by = created_by
        self.created_dt = created_dt
        self.description = description
        self.experiment = experiment
        self.id = id
        if metrics_summary is not None:
            self.metrics_summary = metrics_summary
        self.status = status
        self.tags = tags
        self.updated_dt = updated_dt
        self.version = version

    @property
    def artifact_volume_id(self):
        """Gets the artifact_volume_id of this ResponseModelDetail.  # noqa: E501


        :return: The artifact_volume_id of this ResponseModelDetail.  # noqa: E501
        :rtype: int
        """
        return self._artifact_volume_id

    @artifact_volume_id.setter
    def artifact_volume_id(self, artifact_volume_id):
        """Sets the artifact_volume_id of this ResponseModelDetail.


        :param artifact_volume_id: The artifact_volume_id of this ResponseModelDetail.  # noqa: E501
        :type artifact_volume_id: int
        """
        if self.local_vars_configuration.client_side_validation and artifact_volume_id is None:  # noqa: E501
            raise ValueError("Invalid value for `artifact_volume_id`, must not be `None`")  # noqa: E501

        self._artifact_volume_id = artifact_volume_id

    @property
    def created_by(self):
        """Gets the created_by of this ResponseModelDetail.  # noqa: E501


        :return: The created_by of this ResponseModelDetail.  # noqa: E501
        :rtype: ResponseUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ResponseModelDetail.


        :param created_by: The created_by of this ResponseModelDetail.  # noqa: E501
        :type created_by: ResponseUser
        """
        if self.local_vars_configuration.client_side_validation and created_by is None:  # noqa: E501
            raise ValueError("Invalid value for `created_by`, must not be `None`")  # noqa: E501

        self._created_by = created_by

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseModelDetail.  # noqa: E501


        :return: The created_dt of this ResponseModelDetail.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseModelDetail.


        :param created_dt: The created_dt of this ResponseModelDetail.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def description(self):
        """Gets the description of this ResponseModelDetail.  # noqa: E501


        :return: The description of this ResponseModelDetail.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ResponseModelDetail.


        :param description: The description of this ResponseModelDetail.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def experiment(self):
        """Gets the experiment of this ResponseModelDetail.  # noqa: E501


        :return: The experiment of this ResponseModelDetail.  # noqa: E501
        :rtype: ResponseSimpleExperiment
        """
        return self._experiment

    @experiment.setter
    def experiment(self, experiment):
        """Sets the experiment of this ResponseModelDetail.


        :param experiment: The experiment of this ResponseModelDetail.  # noqa: E501
        :type experiment: ResponseSimpleExperiment
        """
        if self.local_vars_configuration.client_side_validation and experiment is None:  # noqa: E501
            raise ValueError("Invalid value for `experiment`, must not be `None`")  # noqa: E501

        self._experiment = experiment

    @property
    def id(self):
        """Gets the id of this ResponseModelDetail.  # noqa: E501


        :return: The id of this ResponseModelDetail.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseModelDetail.


        :param id: The id of this ResponseModelDetail.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def metrics_summary(self):
        """Gets the metrics_summary of this ResponseModelDetail.  # noqa: E501


        :return: The metrics_summary of this ResponseModelDetail.  # noqa: E501
        :rtype: OrmExperimentMetricsSummary
        """
        return self._metrics_summary

    @metrics_summary.setter
    def metrics_summary(self, metrics_summary):
        """Sets the metrics_summary of this ResponseModelDetail.


        :param metrics_summary: The metrics_summary of this ResponseModelDetail.  # noqa: E501
        :type metrics_summary: OrmExperimentMetricsSummary
        """

        self._metrics_summary = metrics_summary

    @property
    def status(self):
        """Gets the status of this ResponseModelDetail.  # noqa: E501


        :return: The status of this ResponseModelDetail.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseModelDetail.


        :param status: The status of this ResponseModelDetail.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def tags(self):
        """Gets the tags of this ResponseModelDetail.  # noqa: E501


        :return: The tags of this ResponseModelDetail.  # noqa: E501
        :rtype: list[ResponseTagResponse]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this ResponseModelDetail.


        :param tags: The tags of this ResponseModelDetail.  # noqa: E501
        :type tags: list[ResponseTagResponse]
        """
        if self.local_vars_configuration.client_side_validation and tags is None:  # noqa: E501
            raise ValueError("Invalid value for `tags`, must not be `None`")  # noqa: E501

        self._tags = tags

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseModelDetail.  # noqa: E501


        :return: The updated_dt of this ResponseModelDetail.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseModelDetail.


        :param updated_dt: The updated_dt of this ResponseModelDetail.  # noqa: E501
        :type updated_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_dt`, must not be `None`")  # noqa: E501

        self._updated_dt = updated_dt

    @property
    def version(self):
        """Gets the version of this ResponseModelDetail.  # noqa: E501


        :return: The version of this ResponseModelDetail.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ResponseModelDetail.


        :param version: The version of this ResponseModelDetail.  # noqa: E501
        :type version: int
        """
        if self.local_vars_configuration.client_side_validation and version is None:  # noqa: E501
            raise ValueError("Invalid value for `version`, must not be `None`")  # noqa: E501

        self._version = version

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseModelDetail):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseModelDetail):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmWorkspace(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'edges': 'OrmWorkspaceEdges',
        'id': 'int',
        'immutable_slug': 'str',
        'init_script': 'str',
        'last_backup_dt': 'datetime',
        'name': 'str',
        'ports': 'dict(str, object)',
        'updated_dt': 'datetime',
        'workspace_backup_volume': 'int',
        'workspace_created_by': 'int',
        'workspace_organization': 'int'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'edges': 'edges',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'init_script': 'init_script',
        'last_backup_dt': 'last_backup_dt',
        'name': 'name',
        'ports': 'ports',
        'updated_dt': 'updated_dt',
        'workspace_backup_volume': 'workspace_backup_volume',
        'workspace_created_by': 'workspace_created_by',
        'workspace_organization': 'workspace_organization'
    }

    def __init__(self, created_dt=None, edges=None, id=None, immutable_slug=None, init_script=None, last_backup_dt=None, name=None, ports=None, updated_dt=None, workspace_backup_volume=None, workspace_created_by=None, workspace_organization=None, local_vars_configuration=None):  # noqa: E501
        """OrmWorkspace - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._edges = None
        self._id = None
        self._immutable_slug = None
        self._init_script = None
        self._last_backup_dt = None
        self._name = None
        self._ports = None
        self._updated_dt = None
        self._workspace_backup_volume = None
        self._workspace_created_by = None
        self._workspace_organization = None
        self.discriminator = None

        if created_dt is not None:
            self.created_dt = created_dt
        if edges is not None:
            self.edges = edges
        if id is not None:
            self.id = id
        if immutable_slug is not None:
            self.immutable_slug = immutable_slug
        self.init_script = init_script
        self.last_backup_dt = last_backup_dt
        if name is not None:
            self.name = name
        if ports is not None:
            self.ports = ports
        if updated_dt is not None:
            self.updated_dt = updated_dt
        if workspace_backup_volume is not None:
            self.workspace_backup_volume = workspace_backup_volume
        if workspace_created_by is not None:
            self.workspace_created_by = workspace_created_by
        if workspace_organization is not None:
            self.workspace_organization = workspace_organization

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmWorkspace.  # noqa: E501


        :return: The created_dt of this OrmWorkspace.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmWorkspace.


        :param created_dt: The created_dt of this OrmWorkspace.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def edges(self):
        """Gets the edges of this OrmWorkspace.  # noqa: E501


        :return: The edges of this OrmWorkspace.  # noqa: E501
        :rtype: OrmWorkspaceEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmWorkspace.


        :param edges: The edges of this OrmWorkspace.  # noqa: E501
        :type edges: OrmWorkspaceEdges
        """

        self._edges = edges

    @property
    def id(self):
        """Gets the id of this OrmWorkspace.  # noqa: E501


        :return: The id of this OrmWorkspace.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmWorkspace.


        :param id: The id of this OrmWorkspace.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this OrmWorkspace.  # noqa: E501


        :return: The immutable_slug of this OrmWorkspace.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this OrmWorkspace.


        :param immutable_slug: The immutable_slug of this OrmWorkspace.  # noqa: E501
        :type immutable_slug: str
        """

        self._immutable_slug = immutable_slug

    @property
    def init_script(self):
        """Gets the init_script of this OrmWorkspace.  # noqa: E501


        :return: The init_script of this OrmWorkspace.  # noqa: E501
        :rtype: str
        """
        return self._init_script

    @init_script.setter
    def init_script(self, init_script):
        """Sets the init_script of this OrmWorkspace.


        :param init_script: The init_script of this OrmWorkspace.  # noqa: E501
        :type init_script: str
        """

        self._init_script = init_script

    @property
    def last_backup_dt(self):
        """Gets the last_backup_dt of this OrmWorkspace.  # noqa: E501


        :return: The last_backup_dt of this OrmWorkspace.  # noqa: E501
        :rtype: datetime
        """
        return self._last_backup_dt

    @last_backup_dt.setter
    def last_backup_dt(self, last_backup_dt):
        """Sets the last_backup_dt of this OrmWorkspace.


        :param last_backup_dt: The last_backup_dt of this OrmWorkspace.  # noqa: E501
        :type last_backup_dt: datetime
        """

        self._last_backup_dt = last_backup_dt

    @property
    def name(self):
        """Gets the name of this OrmWorkspace.  # noqa: E501


        :return: The name of this OrmWorkspace.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrmWorkspace.


        :param name: The name of this OrmWorkspace.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def ports(self):
        """Gets the ports of this OrmWorkspace.  # noqa: E501


        :return: The ports of this OrmWorkspace.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._ports

    @ports.setter
    def ports(self, ports):
        """Sets the ports of this OrmWorkspace.


        :param ports: The ports of this OrmWorkspace.  # noqa: E501
        :type ports: dict(str, object)
        """

        self._ports = ports

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmWorkspace.  # noqa: E501


        :return: The updated_dt of this OrmWorkspace.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmWorkspace.


        :param updated_dt: The updated_dt of this OrmWorkspace.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def workspace_backup_volume(self):
        """Gets the workspace_backup_volume of this OrmWorkspace.  # noqa: E501


        :return: The workspace_backup_volume of this OrmWorkspace.  # noqa: E501
        :rtype: int
        """
        return self._workspace_backup_volume

    @workspace_backup_volume.setter
    def workspace_backup_volume(self, workspace_backup_volume):
        """Sets the workspace_backup_volume of this OrmWorkspace.


        :param workspace_backup_volume: The workspace_backup_volume of this OrmWorkspace.  # noqa: E501
        :type workspace_backup_volume: int
        """

        self._workspace_backup_volume = workspace_backup_volume

    @property
    def workspace_created_by(self):
        """Gets the workspace_created_by of this OrmWorkspace.  # noqa: E501


        :return: The workspace_created_by of this OrmWorkspace.  # noqa: E501
        :rtype: int
        """
        return self._workspace_created_by

    @workspace_created_by.setter
    def workspace_created_by(self, workspace_created_by):
        """Sets the workspace_created_by of this OrmWorkspace.


        :param workspace_created_by: The workspace_created_by of this OrmWorkspace.  # noqa: E501
        :type workspace_created_by: int
        """

        self._workspace_created_by = workspace_created_by

    @property
    def workspace_organization(self):
        """Gets the workspace_organization of this OrmWorkspace.  # noqa: E501


        :return: The workspace_organization of this OrmWorkspace.  # noqa: E501
        :rtype: int
        """
        return self._workspace_organization

    @workspace_organization.setter
    def workspace_organization(self, workspace_organization):
        """Sets the workspace_organization of this OrmWorkspace.


        :param workspace_organization: The workspace_organization of this OrmWorkspace.  # noqa: E501
        :type workspace_organization: int
        """

        self._workspace_organization = workspace_organization

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmWorkspace):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmWorkspace):
            return True

        return self.to_dict() != other.to_dict()
